unit kuzel_f;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  GL, GLU, ExtCtrls, StdCtrls, Menus;

type
  TForm1 = class(TForm)
    Timer: TTimer;
    Edit1: TEdit;
    MainMenu1: TMainMenu;
    Pomoc1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure Pomoc1Click(Sender: TObject);

  private
    DC: HDC;
    hrc: HGLRC;
    Palette: HPALETTE;

    procedure Kuzel;
    procedure Svetlo1;
    procedure Svetlo2;
    procedure SetDCPixelFormat;

  protected
    procedure Kreslit(var Msg: TWMPaint); message WM_PAINT;
    procedure NovaBarevnaPaleta(var Msg: TWMQueryNewPalette); message WM_QUERYNEWPALETTE;
    procedure ZmenaBarevnePalety(var Msg: TWMPaletteChanged); message WM_PALETTECHANGED;

  public
    X,Y,Z: float;
    UhelX,UhelY,UhelZ: integer;
  end;

var
  Form1: TForm1;

implementation


{$R *.DFM}

var
  UhelX2: integer;


procedure TForm1.SetDCPixelFormat;      //Nastaveni pixelu
var
  hHeap: THandle;
  Barvy, i: Integer;
  Paleta: PLogPalette;
  Cervena, Zelena, Modra: Byte;
  nPixelFormat: Integer;
  pfd: TPixelFormatDescriptor;

begin
  FillChar(pfd, SizeOf(pfd), 0);

  with pfd do begin
    nSize     := sizeof(pfd);                               // Velikost struktury pixelu
    nVersion  := 1;                                         // slo verze
    dwFlags   := PFD_DRAW_TO_WINDOW or
                 PFD_SUPPORT_OPENGL or
                 PFD_DOUBLEBUFFER;                          // struktura pixelu
    iPixelType:= PFD_TYPE_RGBA;                             // RGBA pixelov hodnoty
    cColorBits:= 24;                                        // 24-bit barva
    cDepthBits:= 32;                                        // 32-bit depth buffer
    iLayerType:= PFD_MAIN_PLANE;                            // typ vrstvy
  end;

  nPixelFormat := ChoosePixelFormat(DC, @pfd);
  SetPixelFormat(DC, nPixelFormat, @pfd);

  DescribePixelFormat(DC, nPixelFormat, sizeof(TPixelFormatDescriptor), pfd);

  if ((pfd.dwFlags and PFD_NEED_PALETTE) <> 0) then begin
    Barvy   := 1 shl pfd.cColorBits;
    hHeap     := GetProcessHeap;
    Paleta := HeapAlloc(hHeap, 0, sizeof(TLogPalette) + (Barvy * sizeof(TPaletteEntry)));

    Paleta^.palVersion := $300;
    Paleta^.palNumEntries := Barvy;

    Cervena   := (1 shl pfd.cRedBits) - 1;
    Zelena := (1 shl pfd.cGreenBits) - 1;
    Modra  := (1 shl pfd.cBlueBits) - 1;

    for i := 0 to Barvy - 1 do begin
      Paleta^.palPalEntry[i].peRed   := (((i shr pfd.cRedShift)   and Cervena)   * 255) DIV Cervena;
      Paleta^.palPalEntry[i].peGreen := (((i shr pfd.cGreenShift) and Zelena) * 255) DIV Zelena;
      Paleta^.palPalEntry[i].peBlue  := (((i shr pfd.cBlueShift)  and Modra)  * 255) DIV Modra;
      Paleta^.palPalEntry[i].peFlags := 0;
    end;

    Palette := CreatePalette(Paleta^);
    HeapFree(hHeap, 0, Paleta);

    if (Palette <> 0) then begin
      SelectPalette(DC, Palette, False);
      RealizePalette(DC);
    end;
  end;
end;



procedure TForm1.Svetlo1;
const
  glfLightAmbient : Array[0..3] of GLfloat = (0.1, 0.1, 0.1, 1.0);  //GL_AMBIENT = ambientn svtlo, svtlo kter z i kdy jsou svtla vypnuta
  glfLightDiffuse : Array[0..3] of GLfloat = (1.0, 1.0, 0.0, 1.0);  //GL_DIFFUSE = barva svtla, kter zdroj vyzauje do okol
  glfLightSpecular: Array[0..3] of GLfloat = (0.1, 0.0, 0.0, 1.0);  //GL_SPECULAR = zrcadlov odraz svtla
  glfLightShininess: GLFloat = 127;                                 //GL_SHININESS = intenzita svtelnho odlesku
  glflightPosition: Array[0..3] of GLfloat = (-2.0,-2.0,1.0,1.0);   //GL_Position = pozice svtelnho bodu v prostoru
  glfSpotDirection: Array[0..3] of GLfloat = (2.0,2.0,-1.0,1.0);    // GL_SPOT_DIRECTION = zadn smru svtelnho kuele
  glfLightEmission: Array[0..3] of GLfloat = (0.0,0.0,0.0,1.0);     // svtlo, kter vyzauje urit objekt, nepsob jako zdroj svtla
  glfLinearAttenuation: GLFloat = 0.2;                              // GL_LINEAR_ATTENUATION = linern slbnut intenzity svtla
  glfSpotExponent: GLFloat = 1;                                     //GL_SPOT_EXPONENT = charakterizuje zpsob rozptylu svtla pi odrazu
  glfSpotCutoff:GLFloat = 20;                                       //GL_SPOT_CUTOFF = hel, nm se kuel svtla rozevr od sv osy



begin
  glEnable(GL_DEPTH_TEST);

  glLightfv(GL_LIGHT1, GL_AMBIENT, @glfLightAmbient);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, @glfLightDiffuse);
  glLightfv(GL_LIGHT1, GL_SPECULAR, @glfLightSpecular);
  glLightfv(GL_LIGHT1, GL_SHININESS, @glfLightShininess);
  glLightfv(GL_LIGHT1, GL_POSITION,@glfLightPosition);
  glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, @glfSpotDirection);
  glLightfv(GL_LIGHT1, GL_EMISSION, @glfLightEmission);
  glLightfv(GL_LIGHT1, GL_LINEAR_ATTENUATION, @glfLinearAttenuation);
  glLightfv(GL_LIGHT1, GL_SPOT_EXPONENT, @glfSpotExponent);
  glLightfv(GL_LIGHT1, GL_SPOT_CUTOFF, @glfSpotCutoff);




  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT1);  // prvn svtlo



end;


procedure TForm1.Svetlo2;
const
  glfLightAmbient : Array[0..3] of GLfloat = (0.1, 0.1, 0.1, 1.0);   //GL_AMBIENT = ambientn svtlo, svtlo kter z i kdy jsou svtla vypnuta
  glfLightDiffuse : Array[0..3] of GLfloat = (0.4, 0.4, 0.4, 1.0);   //GL_DIFFUSE = barva svtla, kter zdroj vyzauje do okol
  glfLightSpecular: Array[0..3] of GLfloat = (0.1, 0.1, 0.1, 1.0);   //GL_SPECULAR = zrcadlov odraz svtla
  glfLightShininess: GLFloat = 10;                                   //GL_SHININESS = intenzita svtelnho odlesku
  glflightPosition: Array[0..3] of GLfloat = (0.0,0.0,1.0,1.0);      //GL_Position = pozice svtelnho bodu v prostoru
  glfSpotDirection: Array[0..3] of GLfloat = (0.0,0.0,-1.0,1.0);     // GL_SPOT_DIRECTION = zadn smru svtelnho kuele
  glfLightEmission: Array[0..3] of GLfloat = (0.0,0.0,0.0,1.0);      // svtlo, kter vyzauje urit objekt, nepsob jako zdroj svtla
  glfLinearAttenuation: GLFloat = 0;                                 // GL_LINEAR_ATTENUATION = linern slbnut intenzity svtla
  glfSpotExponent: GLFloat = 1;                                      //GL_SPOT_EXPONENT = charakterizuje zpsob rozptylu svtla pi odrazu
  glfSpotCutoff:GLFloat = 180;                                       //GL_SPOT_CUTOFF = hel, nm se kuel svtla rozevr od sv osy



begin
  glEnable(GL_DEPTH_TEST);

  glLightfv(GL_LIGHT2, GL_AMBIENT, @glfLightAmbient);
  glLightfv(GL_LIGHT2, GL_DIFFUSE, @glfLightDiffuse);
  glLightfv(GL_LIGHT2, GL_SPECULAR, @glfLightSpecular);
  glLightfv(GL_LIGHT2, GL_SHININESS, @glfLightShininess);
  glLightfv(GL_LIGHT2, GL_POSITION, @glfLightPosition);
  glLightfv(GL_LIGHT2, GL_SPOT_DIRECTION, @glfSpotDirection);
  glLightfv(GL_LIGHT2, GL_EMISSION, @glfLightEmission);
  glLightfv(GL_LIGHT2, GL_LINEAR_ATTENUATION, @glfLinearAttenuation);
  glLightfv(GL_LIGHT2, GL_SPOT_EXPONENT, @glfSpotExponent);
  glLightfv(GL_LIGHT2, GL_SPOT_CUTOFF, @glfSpotCutoff);



  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT2);  // druh svtlo
end;


procedure TForm1.FormCreate(Sender: TObject);
begin
  UhelX := 0;
  UhelY := 0;
  UhelZ := 0;
  X := 0; Y := 0; Z := -15;
  DC := GetDC(Handle);
  SetDCPixelFormat;
  hrc := wglCreateContext(DC);
  wglMakeCurrent(DC, hrc);
  Svetlo1;
  Svetlo2;
  Timer.Enabled := True;
end;

procedure TForm1.FormResize(Sender: TObject);
var
  PomerStran : GLdouble;

begin
  PomerStran := Width / Height;

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  gluPerspective(30.0, PomerStran, 0.1, 400);
  glViewport(0, 0, Width ,Height);
  InvalidateRect(Handle, nil, False);
end;


procedure TForm1.FormDestroy(Sender: TObject);
begin
  Timer.Enabled := False;
  wglMakeCurrent(0, 0);
  wglDeleteContext(hrc);
  ReleaseDC(Handle, DC);

  if (Palette <> 0) then
    DeleteObject(Palette);
end;

procedure TForm1.Kreslit(var Msg: TWMPaint);
var
  ps : TPaintStruct;
begin
  BeginPaint(Handle, ps);
  Kuzel;
  EndPaint(Handle, ps);
end;

procedure TForm1.NovaBarevnaPaleta(var Msg : TWMQueryNewPalette);
begin
  if (Palette <> 0) then begin
    Msg.Result := RealizePalette(DC);

    if (Msg.Result <> GDI_ERROR) then
      InvalidateRect(Handle, nil, False);
  end;
end;

procedure TForm1.ZmenaBarevnePalety(var Msg : TWMPaletteChanged);
begin
  if ((Palette <> 0) and (THandle(TMessage(Msg).wParam) <> Handle)) then begin
    if (RealizePalette(DC) <> GDI_ERROR) then
      UpdateColors(DC);

    Msg.Result := 0;
  end;
end;


procedure TForm1.TimerTimer(Sender: TObject);
begin
  Svetlo1;        //tato procedur mus bt volna, aby svtlo nechodilo s kamerou
  InvalidateRect(Handle, nil, False);
end;


procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_UP: begin  //ipka nahoru
        Z := Z + ((-sin((UhelY - 90) * (Pi/180))) * 0.1);
        X := X - ((sin((UhelY) * (Pi/180))) * 0.1);
    end;
  end;
  case Key of
    VK_DOWN: begin //ipka dolu
      Z := Z - ((-sin((UhelY - 90) * (Pi/180))) * 0.1);
      X := X + ((sin((UhelY) * (Pi/180))) * 0.1);
    end;
  end;
  case Key of
    VK_RIGHT: begin //prav ipka
        UhelY := UhelY + 5;
    end;
  end;
  case Key of
    VK_LEFT: begin //lev ipka
        UhelY := UhelY - 5;
    end;
  end;
  case Key of   //klvesa END
    VK_END: begin
      UhelX := UhelX + 5;
  end;
  end;
  case Key of  //klvesa HOME
    VK_HOME: begin
      UhelX := UhelX - 5;
  end;
  end;
  case Key of  //klvesa PAGEUP
    VK_PRIOR: begin
      Y := Y - 0.5;
  end;
  end;
  case Key of  //klvesa PAGEDOWN
    VK_NEXT: begin
      Y := Y + 0.5;
  end;
  end;
end;

procedure TForm1.Pomoc1Click(Sender: TObject);
begin
  ShowMessage('Pohyb pomoc ipek' + chr(13) + 'Pohled nahoru = Home' + chr(13) + 'Pohled dolu = End'+ chr(13) + 'Kamera nahoru = PageUp' + chr(13) + 'Kamera dolu = PageDown');
end;

procedure TForm1.Kuzel;
var
  b,g,r: glfloat;
  plast, zakl: PGLUquadricObj;
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);


  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;

  glRotatef(UhelX, 1, 0, 0);			//rotace kamery  X
  glRotatef(UhelY, 0, 1, 0);			//rotace kamery Y
  glRotatef(UhelZ, 0, 0, 1);			//rotace kamery Z
  glTranslatef(X,Y,Z);				//pozice kamery

  r := 0.0;
  g := 0.5;
  b := 1.0;

  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, @b);
  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, @g);
  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, @r);
  glMaterialfv(GL_FRONT, GL_SPECULAR, @b);
  glMaterialfv(GL_FRONT, GL_SPECULAR, @g);
  glMaterialfv(GL_FRONT, GL_SPECULAR, @r);
  glMaterialfv(GL_FRONT, GL_SHININESS, @b);
  glMaterialfv(GL_FRONT, GL_SHININESS, @g);
  glMaterialfv(GL_FRONT, GL_SHININESS, @r);

  if UhelX2 <= 0 then
    UhelX2 := 360
  else
    UhelX2 := UhelX2 - 5;

  plast := gluNewQuadric();                    //vytvo nov objekt (pl᚝ kuele)
  zakl := gluNewQuadric();                     //vytvo nov objekt (zkladnu kuele)
  glRotatef(UhelX2, 1, 0, 0);                  //rotace objektu
  gluQuadricOrientation(plast,GLU_OUTSIDE);    //vnj strana plt odr svtlo
  gluQuadricOrientation(zakl,GLU_INSIDE);      //vnitn strana zkladny kuele odr svtlo
  gluCylinder(plast, 1, 0, 2, 20, 5);          //zobraz pl᚝
  gluDisk(zakl, 0, 1, 20, 1);                  //zobraz zkladnu

  SwapBuffers(DC);

  Edit1.Text := FloatToStr(r)
end;



end.

